#include "lcd.h";
#include "delay.h";


char lcd_reversebyte(char c) {
    char newc=0;
    char oldc=c;
    char lastbit;
    for (char i=0;i<8;i++) {
        lastbit = oldc & 1;
        oldc = oldc >>1;
        newc = newc<<1 + lastbit;
        //cbit = lcd_bit_get(c,i);
        //if (cbit==1) lcd_bit_set(newc,7-i);
    }
    return newc;
}

void lcd_init() {
    delay_ms(15);   // wait 15+ ms from power on
    LCD_RS = 0;
    LCD_RW = 0;
    LCD_EN = 0;
    /* 001DNF00
     * D  0=4bit,1=8bit N 0=1line,1=2line F 0=5x8 dots,1=5x11
     */
    LCD_DATA = lcd_byte(0b00111000);  // function set 1
    LCD_STROBE();
    delay_ms(5);
    LCD_STROBE();           // function set 2 (LCD_DATA same as before)
    delay_100us();          // wait > 100us
    delay_50us();
    LCD_STROBE();           // function set 3 (data already set)
    // function set 4 : display lines and font configuration
    //LCD_DATA = 48+ 8 * LCD_LINES + 4*LCD_FONT;    // 1|DL|N|F|-|-| DL 1=8bit, 0=4bit, N 0=1 line,1=2line, F 0=5x8,1=5x11
    LCD_DATA = lcd_byte(0b00111100);
    LCD_STROBE();
    delay_50us();
    // 1DCB : D = display on/off , C = curson on/off , B = blink on / off
    LCD_DATA = lcd_byte(0b00001100);//8+LCD_DISPLAYON << 2 + LCD_CURSOR<<1 + LCD_CURSOR_LINK;
    LCD_STROBE();
    delay_50us();
    LCD_DATA = lcd_byte(1);    // clear display, min 1.5-2ms time
    LCD_STROBE();
    delay_ms(2);
    LCD_DATA = lcd_byte(4+LCD_CURSORDIR*2+LCD_SHIFT);    // entry mode set
    LCD_STROBE();
    delay_50us();
}
/*  Waits until the LCD display says it's ready to accept new commands
 */
void lcd_busy() {
    unsigned char flag=0;
    LCD_TRIS = 0xFF;
    LCD_RS = 0;
    LCD_RW = 1;
    do {
        LCD_STROBE();
        if (LCD_DATA_REVERSED==0) {
            flag = LCD_DATA >>7;
        } else {
            flag = LCD_DATA & 1;
        }
        delay_1us();
    } while (flag==1);
    LCD_RW = 0;
    LCD_TRIS=0;
}

void lcd_clear() {
    lcd_busy();
    LCD_RS = 0;
    LCD_DATA = lcd_byte(1);
    LCD_STROBE();
    /* operation takes 1.52ms or more. We don't wait because we assume all
     * functions calling the LCD display will have the busy check before doing
     * anything
     */
}

void lcd_home() {
    lcd_busy();
    LCD_RS = 0;
    LCD_DATA = lcd_byte(2);
    LCD_STROBE();
}

void lcd_goto(char x, char y) {
    unsigned char k;
    /* These offsets are known to be valid for 2x16 and 4x16.
     * Should work with 2x20 or 4x20, in worst case you may have to change them
     * a bit, see datasheet of your lcd screen
     */
    if (y==0) k = 0;
    if (y==1) k = 0x40;
    if (y==2) k = 0x10;
    if (y==3) k = 0x50;
    k+=x;
    k+=128;
    lcd_busy();
    LCD_RS = 0;
    LCD_DATA =  lcd_byte(k);
    LCD_STROBE();
}

void lcd_writechar(char c) {
    lcd_busy();
    LCD_RS = 1;
    LCD_RW = 0;
    LCD_DATA = lcd_byte(c);
    LCD_STROBE();
    LCD_RS = 0;
}

void lcd_writestring( const char * c) {
    while(*c) {
        lcd_writechar(*c++);
    }
}

