/* 
 * File:   lcd.h
 * Author: mariush
 *
 * Created on July 9, 2014, 6:46 PM
 */
#include <htc.h>

#ifndef LCD_H
    #define	LCD_H

    #define LCD_RS PORTBbits.RB2
    #define LCD_RW PORTBbits.RB1
    #define LCD_EN PORTBbits.RB0
    #define LCD_LIGHT PORTBbits.RB4

    #define LCD_DATA PORTD
    #define LCD_TRIS TRISD

    #define LCD_MASK_OFF 0x00;  // all pins on PORT A set to output
    #define LCD_MASK_ON  0x80;  // pin 7/LCD_DATA3 set to input

    #define LCD_LINES 1  // 0= 1-line display mode, 1=2-line display mode
    #define LCD_FONT 0  // 0= 5x8 , 1=5x11
    #define LCD_DISPLAYON 1   // 0= keep lcd off at start, 1= start it
    #define LCD_CURSOR 1    // 0= cursor off, 1=cursor on
    #define LCD_CURSORBLINK 0   // 0=blink off, 1= on
    #define LCD_CURSORDIR 1 // 0=left, 1=right
    #define LCD_SHIFT 0 // 0=shift right, 1=shift left

    // keep LCD_EN =1 for 500ns when clock is 16Mhz
    #define	LCD_STROBE()	((LCD_EN = 1),(LCD_EN = 1), (LCD_EN=0))

    #define lcd_bit_get(var,bitno)   ((var) & (1 << (bitno)))
    #define lcd_bit_set(var,bitno)    ((var) |= 1 << (bitno))
    #define lcd_bit_clear(var,bitno)  ((var) &= ~(1 << (bitno)))

    /* Some LCD displays displays have the pins arranged in the order RS,RW,E, D0..D7
     * but most PIC microcontrollers have pins arranged as RD7, RD6 .. RD0
     *
     * Set LCD_DATA_REVERSED to 1, comment lcd_byte define above and uncomment the
     * define below to have the code reverse the data sent on the data port
     * Reversed pins help avoid mess of wires in some situations.
     */

    #define LCD_DATA_REVERSED 0     // 0 means normal order, 1 means reversed
    #define lcd_byte(var) var       // D0..D7 -> D0..D7
    //#define lcd_byte(var) lcd_reversebyte(var) // D0..D7 -> D7.. D0



    extern void lcd_init();
    extern void lcd_busy();
    extern void lcd_clear();
    extern void lcd_home();
    extern void lcd_goto(char x, char y);
    extern void lcd_writechar(char c);
    extern void lcd_writestring( const char * c);
    extern char lcd_reversebyte(char c);
#endif	/* LCD_H */

